#!/usr/bin/python
# Copyright (c) 2020, 2022 Oracle and/or its affiliates.
# This software is made available to you under the terms of the GPL 3.0 license or the Apache 2.0 license.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# Apache License v2.0
# See LICENSE.TXT for details.
# GENERATED FILE - DO NOT EDIT - MANUAL CHANGES WILL BE OVERWRITTEN


from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: oci_adm_vulnerability_audit_actions
short_description: Perform actions on a VulnerabilityAudit resource in Oracle Cloud Infrastructure
description:
    - Perform actions on a VulnerabilityAudit resource in Oracle Cloud Infrastructure
    - For I(action=change_compartment), moves a Vulnerability Audit from one compartment to another.
version_added: "2.9.0"
author: Oracle (@oracle)
options:
    vulnerability_audit_id:
        description:
            - Unique Vulnerability Audit identifier path parameter.
        type: str
        aliases: ["id"]
        required: true
    compartment_id:
        description:
            - The L(OCID,https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the compartment to which the resource must be moved.
        type: str
        required: true
    action:
        description:
            - The action to perform on the VulnerabilityAudit.
        type: str
        required: true
        choices:
            - "change_compartment"
extends_documentation_fragment: [ oracle.oci.oracle ]
"""

EXAMPLES = """
- name: Perform action change_compartment on vulnerability_audit
  oci_adm_vulnerability_audit_actions:
    # required
    vulnerability_audit_id: "ocid1.vulnerabilityaudit.oc1..xxxxxxEXAMPLExxxxxx"
    compartment_id: "ocid1.compartment.oc1..xxxxxxEXAMPLExxxxxx"
    action: change_compartment

"""

RETURN = """
vulnerability_audit:
    description:
        - Details of the VulnerabilityAudit resource acted upon by the current operation
    returned: on success
    type: complex
    contains:
        id:
            description:
                - The Oracle Cloud identifier (L(OCID,https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm)) of the Vulnerability Audit.
            returned: on success
            type: str
            sample: "ocid1.resource.oc1..xxxxxxEXAMPLExxxxxx"
        display_name:
            description:
                - The name of the Vulnerability Audit.
            returned: on success
            type: str
            sample: display_name_example
        knowledge_base_id:
            description:
                - The Oracle Cloud identifier (L(OCID,https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm)) of the Knowledge Base.
            returned: on success
            type: str
            sample: "ocid1.knowledgebase.oc1..xxxxxxEXAMPLExxxxxx"
        time_created:
            description:
                - The creation date and time of the Vulnerability Audit (formatted according to L(RFC3339,https://datatracker.ietf.org/doc/html/rfc3339)).
            returned: on success
            type: str
            sample: "2013-10-20T19:20:30+01:00"
        time_updated:
            description:
                - The update date and time of the Vulnerability Audit (formatted according to L(RFC3339,https://datatracker.ietf.org/doc/html/rfc3339)).
            returned: on success
            type: str
            sample: "2013-10-20T19:20:30+01:00"
        lifecycle_state:
            description:
                - The current lifecycle state of the Vulnerability Audit.
            returned: on success
            type: str
            sample: ACTIVE
        vulnerabilities:
            description:
                - List of vulnerabilities found in the Vulnerability Audit.
            returned: on success
            type: complex
            contains:
                id:
                    description:
                        - Unique vulnerability identifier, e.g. CVE-1999-0067.
                    returned: on success
                    type: str
                    sample: "ocid1.resource.oc1..xxxxxxEXAMPLExxxxxx"
                cvss_v2_score:
                    description:
                        - Common Vulnerability Scoring System (CVSS) Version 2.
                    returned: on success
                    type: float
                    sample: 3.4
                cvss_v3_score:
                    description:
                        - Common Vulnerability Scoring System (CVSS) Version 3.
                    returned: on success
                    type: float
                    sample: 3.4
                is_ignored:
                    description:
                        - Indicates if the vulnerability was ignored according to the audit configuration.
                    returned: on success
                    type: bool
                    sample: true
        max_observed_cvss_v2_score:
            description:
                - Maximum Common Vulnerability Scoring System Version 2 score of vulnerabilities.
            returned: on success
            type: float
            sample: 3.4
        max_observed_cvss_v3_score:
            description:
                - Maximum Common Vulnerability Scoring System Version 3 score of vulnerabilities.
            returned: on success
            type: float
            sample: 3.4
        vulnerable_artifacts_count:
            description:
                - Count of vulnerable artifacts.
            returned: on success
            type: int
            sample: 56
        configuration:
            description:
                - ""
            returned: on success
            type: complex
            contains:
                max_permissible_cvss_v2_score:
                    description:
                        - The minimum CVSS V2 score for a vulnerability to be considered by the audit as failed.
                    returned: on success
                    type: float
                    sample: 3.4
                max_permissible_cvss_v3_score:
                    description:
                        - The minimum CVSS V3 score for a vulnerability to be considered by the audit as failed.
                    returned: on success
                    type: float
                    sample: 3.4
                exclusions:
                    description:
                        - Pattern to exclude artifacts from audit.
                    returned: on success
                    type: list
                    sample: []
        is_success:
            description:
                - Indicates if an audit succeeded according to the configuration. The value is `null` if the audit is in the `CREATING` state.
            returned: on success
            type: bool
            sample: true
        build_type:
            description:
                - The type of the build tool.
            returned: on success
            type: str
            sample: MAVEN
        compartment_id:
            description:
                - The Oracle Cloud identifier (L(OCID,https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm)) of the compartment associated
                  with the Vulnerability Audit.
            returned: on success
            type: str
            sample: "ocid1.compartment.oc1..xxxxxxEXAMPLExxxxxx"
        freeform_tags:
            description:
                - "Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only.
                  Example: `{\\"bar-key\\": \\"value\\"}`"
            returned: on success
            type: dict
            sample: {'Department': 'Finance'}
        defined_tags:
            description:
                - "Defined tags for this resource. Each key is predefined and scoped to a namespace.
                  Example: `{\\"foo-namespace\\": {\\"bar-key\\": \\"value\\"}}`"
            returned: on success
            type: dict
            sample: {'Operations': {'CostCenter': 'US'}}
        system_tags:
            description:
                - "Usage of system tag keys. These predefined keys are scoped to namespaces.
                  Example: `{\\"orcl-cloud\\": {\\"free-tier-retained\\": \\"true\\"}}`"
            returned: on success
            type: dict
            sample: {}
    sample: {
        "id": "ocid1.resource.oc1..xxxxxxEXAMPLExxxxxx",
        "display_name": "display_name_example",
        "knowledge_base_id": "ocid1.knowledgebase.oc1..xxxxxxEXAMPLExxxxxx",
        "time_created": "2013-10-20T19:20:30+01:00",
        "time_updated": "2013-10-20T19:20:30+01:00",
        "lifecycle_state": "ACTIVE",
        "vulnerabilities": [{
            "id": "ocid1.resource.oc1..xxxxxxEXAMPLExxxxxx",
            "cvss_v2_score": 3.4,
            "cvss_v3_score": 3.4,
            "is_ignored": true
        }],
        "max_observed_cvss_v2_score": 3.4,
        "max_observed_cvss_v3_score": 3.4,
        "vulnerable_artifacts_count": 56,
        "configuration": {
            "max_permissible_cvss_v2_score": 3.4,
            "max_permissible_cvss_v3_score": 3.4,
            "exclusions": []
        },
        "is_success": true,
        "build_type": "MAVEN",
        "compartment_id": "ocid1.compartment.oc1..xxxxxxEXAMPLExxxxxx",
        "freeform_tags": {'Department': 'Finance'},
        "defined_tags": {'Operations': {'CostCenter': 'US'}},
        "system_tags": {}
    }
"""

from ansible_collections.oracle.oci.plugins.module_utils import (
    oci_common_utils,
    oci_wait_utils,
)
from ansible_collections.oracle.oci.plugins.module_utils.oci_resource_utils import (
    OCIActionsHelperBase,
    OCIAnsibleModule,
    get_custom_class,
)

try:
    from oci.adm import ApplicationDependencyManagementClient
    from oci.adm.models import ChangeVulnerabilityAuditCompartmentDetails

    HAS_OCI_PY_SDK = True
except ImportError:
    HAS_OCI_PY_SDK = False


class VulnerabilityAuditActionsHelperGen(OCIActionsHelperBase):
    """
    Supported actions:
        change_compartment
    """

    @staticmethod
    def get_module_resource_id_param():
        return "vulnerability_audit_id"

    def get_module_resource_id(self):
        return self.module.params.get("vulnerability_audit_id")

    def get_get_fn(self):
        return self.client.get_vulnerability_audit

    def get_resource(self):
        return oci_common_utils.call_with_backoff(
            self.client.get_vulnerability_audit,
            vulnerability_audit_id=self.module.params.get("vulnerability_audit_id"),
        )

    def change_compartment(self):
        action_details = oci_common_utils.convert_input_data_to_model_class(
            self.module.params, ChangeVulnerabilityAuditCompartmentDetails
        )
        return oci_wait_utils.call_and_wait(
            call_fn=self.client.change_vulnerability_audit_compartment,
            call_fn_args=(),
            call_fn_kwargs=dict(
                vulnerability_audit_id=self.module.params.get("vulnerability_audit_id"),
                change_vulnerability_audit_compartment_details=action_details,
            ),
            waiter_type=oci_wait_utils.NONE_WAITER_KEY,
            operation="{0}_{1}".format(
                self.module.params.get("action").upper(),
                oci_common_utils.ACTION_OPERATION_KEY,
            ),
            waiter_client=self.get_waiter_client(),
            resource_helper=self,
            wait_for_states=self.get_action_desired_states(
                self.module.params.get("action")
            ),
        )


VulnerabilityAuditActionsHelperCustom = get_custom_class(
    "VulnerabilityAuditActionsHelperCustom"
)


class ResourceHelper(
    VulnerabilityAuditActionsHelperCustom, VulnerabilityAuditActionsHelperGen
):
    pass


def main():
    module_args = oci_common_utils.get_common_arg_spec(
        supports_create=False, supports_wait=False
    )
    module_args.update(
        dict(
            vulnerability_audit_id=dict(aliases=["id"], type="str", required=True),
            compartment_id=dict(type="str", required=True),
            action=dict(type="str", required=True, choices=["change_compartment"]),
        )
    )

    module = OCIAnsibleModule(argument_spec=module_args, supports_check_mode=True)

    if not HAS_OCI_PY_SDK:
        module.fail_json(msg="oci python sdk required for this module.")

    resource_helper = ResourceHelper(
        module=module,
        resource_type="vulnerability_audit",
        service_client_class=ApplicationDependencyManagementClient,
        namespace="adm",
    )

    result = resource_helper.perform_action(module.params.get("action"))

    module.exit_json(**result)


if __name__ == "__main__":
    main()
